@@ -32,9 +32,6 @@
 #include "entities/Detector.h"
 #include "entities/Hero.h"
 
-//#include <iostream>
-//#define MAP_PREF_DISP
-
 MapLoader Map::map_loader;
 
 /**
@@ -448,29 +445,15 @@ void Map::check_suspended() {
  * @brief Displays the map with all its entities on the screen.
  */
 void Map::display() {
-#ifdef MAP_PREF_DISP
-  uint32_t a,b,c,d;
-  a = SDL_GetTicks();
-#endif
 
   // background
   visible_surface->fill_with_color(tileset->get_background_color());
 
-#ifdef MAP_PREF_DISP
-  b = SDL_GetTicks();
-#endif
   // display all entities (including the hero)
   entities->display();
 
-#ifdef MAP_PREF_DISP
-  c = SDL_GetTicks();
-#endif
   // foreground
   display_foreground();
-#ifdef MAP_PREF_DISP
-  d = SDL_GetTicks();
-  std::cout << "[Map-Disp] - | 1: " << b-a << " | 2: " << c-b << " | 3: " << d-c << std::endl;
-#endif
 }
 
 /**
@@ -875,8 +858,7 @@ void Map::check_collision_with_detectors(MapEntity &entity) {
        i != detectors.end();
        i++) {
 
-    if (!(*i)->is_being_removed()
-	&& (*i)->is_enabled()) {
+    if (!(*i)->is_being_removed() && (*i)->is_enabled()) {
       (*i)->check_collision(entity);
     }
   }
