@@ -23,6 +23,7 @@
 #include "entities/ConveyorBelt.h"
 #include "entities/Switch.h"
 #include "entities/CrystalSwitch.h"
+#include "entities/Chest.h"
 #include "entities/Block.h"
 #include "entities/JumpSensor.h"
 #include "entities/Sensor.h"
@@ -36,6 +37,7 @@
 #include "hero/GrabbingState.h"
 #include "hero/HurtState.h"
 #include "hero/JumpingState.h"
+#include "hero/ForcedWalkingState.h"
 #include "hero/LiftingState.h"
 #include "hero/PlungingState.h"
 #include "hero/RunningState.h"
@@ -44,11 +46,13 @@
 #include "hero/TreasureState.h"
 #include "hero/VictoryState.h"
 #include "hero/BoomerangState.h"
+#include "hero/HookshotState.h"
 #include "hero/BowState.h"
 #include "movements/RectilinearMovement.h"
 #include "lowlevel/System.h"
 #include "lowlevel/Debug.h"
 #include "lowlevel/StringConcat.h"
+#include "lowlevel/Sound.h"
 #include "Game.h"
 #include "Map.h"
 #include "Equipment.h"
@@ -297,7 +301,7 @@ void Hero::update_ground() {
         if (!get_map().test_collision_with_obstacles(get_layer(), collision_box, *this)) {
           set_bounding_box(collision_box);
           notify_position_changed();
-	  moved = true;
+          moved = true;
         }
 
 	if (!moved && hole_dxy.get_x() != 0) { // try x only
@@ -1012,6 +1016,9 @@ void Hero::check_position() {
         && entities.get_obstacle_tile(layer, x + 15, y + 15) == OBSTACLE_EMPTY) {
 
       get_entities().set_entity_layer(this, Layer(layer - 1));
+      if (state->is_free() && get_tile_ground() == GROUND_NORMAL) {
+        Sound::play("hero_lands");
+      }
     }
   }
 }
@@ -1326,8 +1333,8 @@ bool Hero::is_raised_block_obstacle(CrystalSwitchBlock &raised_block) {
  * @param jump_sensor a non-diagonal jump sensor
  * @return true if the jump sensor is currently an obstacle for this entity
  */
-bool Hero::is_jump_sensor_obstacle(JumpSensor &jump_sensor) {
-  return !state->can_take_jump_sensor(); // if the jump sensors cannot be used in this state, consider them as obstacles
+bool Hero::is_jump_sensor_obstacle(JumpSensor& jump_sensor) {
+  return state->is_jump_sensor_obstacle(jump_sensor);
 }
 
 /**
@@ -1381,7 +1388,7 @@ void Hero::notify_collision_with_teletransporter(Teletransporter &teletransporte
 
   if (teletransporter.is_on_map_side() || !state->can_avoid_teletransporter()) {
 
-    bool on_hole = get_tile_ground() == GROUND_HOLE;
+    bool on_hole = get_ground() == GROUND_HOLE;
     if (on_hole || state->is_teletransporter_delayed()) {
       this->delayed_teletransporter = &teletransporter; // fall into the hole (or do something else) first, transport later
     }
@@ -1490,10 +1497,12 @@ void Hero::notify_collision_with_jump_sensor(JumpSensor &jump_sensor) {
 /**
  * @brief This function is called when a sensor detects a collision with this entity.
  * @param sensor a sensor
+ * @param collision_mode the collision mode that detected the collision
  */
-void Hero::notify_collision_with_sensor(Sensor &sensor) {
+void Hero::notify_collision_with_sensor(Sensor &sensor, CollisionMode collision_mode) {
 
-  if (!state->can_avoid_sensor()) {
+  if (collision_mode == COLLISION_INSIDE    // the hero is entirely inside the sensor
+      && !state->can_avoid_sensor()) {
     sensor.activate(*this);
   }
 }
@@ -1543,6 +1552,36 @@ void Hero::notify_collision_with_crystal_switch(CrystalSwitch &crystal_switch, S
   }
 }
 
+/**
+ * @brief This function is called when a chest detects a collision with this entity.
+ * @param chest the chest
+ */
+void Hero::notify_collision_with_chest(Chest& chest) {
+
+  if (get_keys_effect().get_action_key_effect() == KeysEffect::ACTION_KEY_NONE
+      && is_free()
+      && is_facing_direction4(1)
+      && !chest.is_open()) {
+
+    // we show the 'open' icon, even if this is a big chest and the player does not have the big key
+    get_keys_effect().set_action_key_effect(KeysEffect::ACTION_KEY_OPEN);
+  }
+}
+
+/**
+ * @brief This function is called when a block detects a collision with this entity.
+ * @param block the block
+ */
+void Hero::notify_collision_with_block(Block& block) {
+
+  if (get_keys_effect().get_action_key_effect() == KeysEffect::ACTION_KEY_NONE
+      && is_free()) {
+
+    // we show the action icon
+    get_keys_effect().set_action_key_effect(KeysEffect::ACTION_KEY_GRAB);
+  }
+}
+
 /**
  * @brief This function is called when a bomb detects a collision with this entity.
  * @param bomb the bomb
@@ -1915,6 +1954,22 @@ void Hero::start_treasure(const Treasure &treasure) {
   set_state(new TreasureState(*this, treasure));
 }
 
+/**
+ * @brief Makes the hero walk with a predetermined path.
+ *
+ * The player does not control him anymore.
+ *
+ * @param path the succession of basic moves
+ * composing this movement (each character represents
+ * a direction between '0' and '7'), as in PathMovement
+ * @param loop true to make the movement return to the beginning
+ * once finished
+ * @param ignore_obstacles true to make the movement ignore obstacles
+ */
+void Hero::start_forced_walking(const std::string &path, bool loop, bool ignore_obstacles) {
+  set_state(new ForcedWalkingState(*this, path, loop, ignore_obstacles));
+}
+
 /**
  * @brief Makes the hero jump into a direction.
  *
@@ -1987,6 +2042,13 @@ void Hero::start_bow() {
   set_state(new BowState(*this));
 }
 
+/**
+ * @brief Starts shooting the hookshot.
+ */
+void Hero::start_hookshot() {
+  set_state(new HookshotState(*this));
+}
+
 /**
  * @brief Activates immediately the state corresponding to the current ground.
  *
