@@ -25,18 +25,17 @@
 #include "entities/Boomerang.h"
 #include "Map.h"
 #include "Game.h"
+#include "lowlevel/Surface.h"
+#include "lowlevel/Color.h"
 #include "lowlevel/Music.h"
 #include "lowlevel/Debug.h"
 #include "lowlevel/StringConcat.h"
 using std::list;
 
-//#include <iostream>
-//#define MAPENT_PERF_DISP
-
 /**
  * @brief Constructor.
  * @param game the game
- * @param map the map
+ * @param map the map (not loaded yet)
  */
 MapEntities::MapEntities(Game &game, Map &map):
   game(game),
@@ -48,13 +47,23 @@ MapEntities::MapEntities(Game &game, Map &map):
   this->obstacle_entities[layer].push_back(&hero);
   this->entities_displayed_y_order[layer].push_back(&hero);
   // TODO update that when the layer changes, same thing for enemies
+
+  // surfaces to pre-render static tiles
+  for (int layer = 0; layer < LAYER_NB; layer++) {
+    non_animated_tiles_surfaces[layer] = NULL;
+  }
 }
 
 /**
  * @brief Destructor.
  */
 MapEntities::~MapEntities() {
+
   destroy_all_entities();
+
+  for (int layer = 0; layer < LAYER_NB; layer++) {
+    delete non_animated_tiles_surfaces[layer];
+  }
 }
 
 /**
@@ -258,6 +267,9 @@ void MapEntities::notify_map_started() {
     MapEntity *entity = *i;
     entity->notify_map_started();
   }
+
+  // pre-render non-animated tiles
+  build_non_animated_tiles();
 }
 
 /**
@@ -286,12 +298,6 @@ void MapEntities::add_tile(Tile *tile) {
 
   int i, j;
  
-  for (i = 0; i < tile_height8; i++) {
-    for (j = 0; j < tile_width8; j++) {
-      set_obstacle(layer, tile_x8 + j, tile_y8 + i, OBSTACLE_NONE);
-    }
-  }
-
   switch (obstacle) {
 
     /* If the obstacle property is the same for all points inside the base tile,
@@ -307,7 +313,7 @@ void MapEntities::add_tile(Tile *tile) {
   case OBSTACLE:
     for (i = 0; i < tile_height8; i++) {
       for (j = 0; j < tile_width8; j++) {
-	set_obstacle(layer, tile_x8 + j, tile_y8 + i, obstacle);
+        set_obstacle(layer, tile_x8 + j, tile_y8 + i, obstacle);
       }
     }
     break;
@@ -324,9 +330,14 @@ void MapEntities::add_tile(Tile *tile) {
       // 8*8 square on the diagonal
       set_obstacle(layer, tile_x8 + i, tile_y8 + i, OBSTACLE_TOP_RIGHT);
 
+      // left part of the row: we are in the bottom-left corner
+      for (j = 0; j < i; j++) {
+        set_obstacle(layer, tile_x8 + j, tile_y8 + i, OBSTACLE_NONE);
+      }
+
       // right part of the row: we are in the top-right corner
       for (j = i + 1; j < tile_width8; j++) {
-	set_obstacle(layer, tile_x8 + j, tile_y8 + i, OBSTACLE);
+        set_obstacle(layer, tile_x8 + j, tile_y8 + i, OBSTACLE);
       }
     }
     break;
@@ -335,9 +346,14 @@ void MapEntities::add_tile(Tile *tile) {
     // we traverse each row of 8*8 squares on the tile
     for (i = 0; i < tile_height8; i++) {
 
+      // right part of the row: we are in the bottom-right corner
+      for (j = tile_width8 - i; j < tile_width8; j++) {
+        set_obstacle(layer, tile_x8 + j, tile_y8 + i, OBSTACLE_NONE);
+      }
+
       // left part of the row: we are in the top-left corner
       for (j = 0; j < tile_width8 - i - 1; j++) {
-	set_obstacle(layer, tile_x8 + j, tile_y8 + i, OBSTACLE);
+        set_obstacle(layer, tile_x8 + j, tile_y8 + i, OBSTACLE);
       }
 
       // 8*8 square on the diagonal
@@ -349,9 +365,13 @@ void MapEntities::add_tile(Tile *tile) {
     // we traverse each row of 8*8 squares on the tile
     for (i = 0; i < tile_height8; i++) {
 
+      // right part of the row: we are in the top-right corner
+      for (j = i + 1; j < tile_width8; j++) {
+        set_obstacle(layer, tile_x8 + j, tile_y8 + i, OBSTACLE_NONE);
+      }
       // left part of the row: we are in the bottom-left corner
       for (j = 0; j < i; j++) {
-	set_obstacle(layer, tile_x8 + j, tile_y8 + i, OBSTACLE);
+        set_obstacle(layer, tile_x8 + j, tile_y8 + i, OBSTACLE);
       }
 
       // 8*8 square on the diagonal
@@ -366,15 +386,20 @@ void MapEntities::add_tile(Tile *tile) {
       // 8*8 square on the diagonal
       set_obstacle(layer, tile_x8 + tile_width8 - i - 1, tile_y8 + i, OBSTACLE_BOTTOM_RIGHT);
 
+      // left part of the row: we are in the top-left corner
+      for (j = 0; j < tile_width8 - i - 1; j++) {
+        set_obstacle(layer, tile_x8 + j, tile_y8 + i, OBSTACLE_NONE);
+      }
+
       // right part of the row: we are in the bottom-right corner
       for (j = tile_width8 - i; j < tile_width8; j++) {
-	set_obstacle(layer, tile_x8 + j, tile_y8 + i, OBSTACLE);
+        set_obstacle(layer, tile_x8 + j, tile_y8 + i, OBSTACLE);
       }
     }
     break;
 
   case OBSTACLE_EMPTY:
-    Debug::die("Illegal obstacle property for this tile");
+    // keep the obstacle property from any tile already here
     break;
   }
 }
@@ -601,6 +626,103 @@ void MapEntities::update() {
   remove_marked_entities();
 }
 
+/**
+ * @brief Draws all non-animated tiles on intermediate surfaces.
+ *
+ * They are drawn only once and then these surfaces are displayed on the screen.
+ */
+void MapEntities::build_non_animated_tiles() {
+
+  const Rectangle map_size(0, 0, map.get_width(), map.get_height());
+  for (int layer = 0; layer < LAYER_NB; layer++) {
+
+    non_animated_tiles_surfaces[layer] = new Surface(map_size.get_width(), map_size.get_height());
+    non_animated_tiles_surfaces[layer]->set_transparency_color(Color::get_magenta());
+    non_animated_tiles_surfaces[layer]->fill_with_color(Color::get_magenta());
+
+    for (unsigned int i = 0; i < tiles[layer].size(); i++) {
+      Tile& tile = *tiles[layer][i];
+      if (!tile.is_animated()) {
+        // non-animated tile: optimize its displaying
+        tile.display(non_animated_tiles_surfaces[layer], map_size);
+      }
+      else {
+        // animated tile: mark its region as non-optimizable
+        // (otherwise, a non-animated tile above an animated one would screw us)
+
+        int tile_x8 = tile.get_x() / 8;
+        int tile_y8 = tile.get_y() / 8;
+        int tile_width8 = tile.get_width() / 8;
+        int tile_height8 = tile.get_height() / 8;
+
+        for (int i = 0; i < tile_height8; i++) {
+          for (int j = 0; j < tile_width8; j++) {
+
+            int x8 = tile_x8 + j;
+            int y8 = tile_y8 + i;
+            if (x8 >= 0 && x8 < map_width8 && y8 >= 0 && y8 < map_height8) {
+              int index = y8 * map_width8 + x8;
+              animated_tiles[layer][index] = true;
+            }
+          }
+        }
+      }
+    }
+
+    // erase the rectangles that contain animated tiles
+    int index = 0;
+    for (int y = 0; y < map.get_height(); y += 8) {
+      for (int x = 0; x < map.get_width(); x += 8) {
+
+        if (animated_tiles[layer][index]) {
+          Rectangle animated_square(x, y, 8, 8);
+          non_animated_tiles_surfaces[layer]->fill_with_color(Color::get_magenta(), animated_square);
+        }
+        index++;
+      }
+    }
+
+    // build the list of animated tiles and tiles overlapping them
+    for (unsigned int i = 0; i < tiles[layer].size(); i++) {
+      Tile& tile = *tiles[layer][i];
+      if (tile.is_animated() || overlaps_animated_tile(tile)) {
+        tiles_in_animated_regions[layer].push_back(&tile);
+      }
+    }
+  }
+}
+
+/**
+ * @brief Returns whether a tile is overlapping an animated other tile.
+ * @param tile the tile to check
+ * @return true if this tile is overlapping an animated tile
+ */
+bool MapEntities::overlaps_animated_tile(Tile& tile) {
+
+  bool* animated_tiles_layer = animated_tiles[tile.get_layer()];
+
+  int tile_x8 = tile.get_x() / 8;
+  int tile_y8 = tile.get_y() / 8;
+  int tile_width8 = tile.get_width() / 8;
+  int tile_height8 = tile.get_height() / 8;
+
+  for (int i = 0; i < tile_height8; i++) {
+    for (int j = 0; j < tile_width8; j++) {
+
+      int x8 = tile_x8 + j;
+      int y8 = tile_y8 + i;
+      if (x8 >= 0 && x8 < map_width8 && y8 >= 0 && y8 < map_height8) {
+
+        int index = y8 * map_width8 + x8;
+        if (animated_tiles_layer[index]) {
+          return true;
+        }
+      }
+    }
+  }
+  return false;
+}
+
 /**
  * @brief Displays the entities on the map surface.
  */
@@ -608,36 +730,35 @@ void MapEntities::display() {
 
   for (int layer = 0; layer < LAYER_NB; layer++) {
 
-    // put the tiles
-#ifdef MAPENT_PERF_DISP
-   std::cout << " [" << layer << "] = " << tiles[layer].size();
-#endif
-    for (unsigned int i = 0; i < tiles[layer].size(); i++) {
-      tiles[layer][i]->display_on_map();
+    // draw the animated tiles and the tiles that overlap them:
+    // in other words, draw all regions containing animated tiles
+    // (and maybe more, but we don't care because non-animated tiles
+    // will be displayed later)
+    for (unsigned int i = 0; i < tiles_in_animated_regions[layer].size(); i++) {
+      tiles_in_animated_regions[layer][i]->display_on_map();
     }
 
-    // put the first sprites
+    // draw the non-animated tiles (with transparent rectangles on the regions of animated tiles
+    // since they are already drawn)
+    non_animated_tiles_surfaces[layer]->blit(map.get_camera_position(), map.get_visible_surface());
+
+    // draw the first sprites
     list<MapEntity*>::iterator i;
     for (i = entities_displayed_first[layer].begin();
 	 i != entities_displayed_first[layer].end();
 	 i++) {
 
       MapEntity *entity = *i;
-      /*
-      Debug::check_assertion(entity->get_layer() == layer,
-	  StringConcat() << "Trying to display entity " << entity << " on layer "
-	  << layer << " but it is actually on layer " << entity->get_layer());
-      */
       if (entity->is_enabled()) {
         entity->display_on_map();
       }
     }
 
-    // put the sprites displayed at the hero's level, in the order
+    // draw the sprites displayed at the hero's level, in the order
     // defined by their y position (including the hero)
     for (i = entities_displayed_y_order[layer].begin();
-	 i != entities_displayed_y_order[layer].end();
-	 i++) {
+	  i != entities_displayed_y_order[layer].end();
+	  i++) {
 
       MapEntity *entity = *i;
       if (entity->is_enabled()) {
@@ -645,9 +766,6 @@ void MapEntities::display() {
       }
     }
   }
-#ifdef MAPENT_PERF_DISP
-   std::cout << std::endl;
-#endif
 }
 
 /**
@@ -703,7 +821,7 @@ void MapEntities::set_entity_layer(MapEntity *entity, Layer layer) {
  * @param rectangle a rectangle
  * @return true if this rectangle overlaps a raised crystal switch block
  */
-bool MapEntities::overlaps_raised_blocks(Layer layer, const Rectangle &rectangle) {
+bool MapEntities::overlaps_raised_blocks(Layer layer, const Rectangle& rectangle) {
 
   bool overlaps = false;
   std::list<CrystalSwitchBlock*> blocks = get_crystal_switch_blocks(layer);
